@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2015 Live Networks, Inc.  All rights reserved.
 // 'Group sockets'
 // Implementation
 
@@ -31,12 +31,12 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 OutputSocket::OutputSocket(UsageEnvironment& env)
   : Socket(env, 0 /* let kernel choose port */),
-    fSourcePort(0), fLastSentTTL(0) {
+    fSourcePort(0), fLastSentTTL(256/*hack: a deliberately invalid value*/) {
 }
 
 OutputSocket::OutputSocket(UsageEnvironment& env, Port port)
   : Socket(env, port),
-    fSourcePort(0), fLastSentTTL(0) {
+    fSourcePort(0), fLastSentTTL(256/*hack: a deliberately invalid value*/) {
 }
 
 OutputSocket::~OutputSocket() {
@@ -44,16 +44,14 @@ OutputSocket::~OutputSocket() {
 
 Boolean OutputSocket::write(netAddressBits address, Port port, u_int8_t ttl,
 			    unsigned char* buffer, unsigned bufferSize) {
-  if (ttl == fLastSentTTL) {
-    // Optimization: So we don't do a 'set TTL' system call again
-    ttl = 0;
-  } else {
-    fLastSentTTL = ttl;
-  }
   struct in_addr destAddr; destAddr.s_addr = address;
-  if (!writeSocket(env(), socketNum(), destAddr, port, ttl,
-		   buffer, bufferSize))
-    return False;
+  if ((unsigned)ttl == fLastSentTTL) {
+    // Optimization: Don't do a 'set TTL' system call again
+    if (!writeSocket(env(), socketNum(), destAddr, port, buffer, bufferSize)) return False;
+  } else {
+    if (!writeSocket(env(), socketNum(), destAddr, port, ttl, buffer, bufferSize)) return False;
+    fLastSentTTL = (unsigned)ttl;
+  }
 
   if (sourcePortNum() == 0) {
     // Now that we've sent a packet, we can find out what the
@@ -462,11 +460,12 @@ UsageEnvironment& operator<<(UsageEnvironment& s, const Groupsock& g) {
 
 // A hash table used to index Groupsocks by socket number.
 
-static HashTable* getSocketTable(UsageEnvironment& env) {
-  if (env.groupsockPriv == NULL) { // We need to create it
-    env.groupsockPriv = HashTable::create(ONE_WORD_HASH_KEYS);
+static HashTable*& getSocketTable(UsageEnvironment& env) {
+  _groupsockPriv* priv = groupsockPriv(env);
+  if (priv->socketTable == NULL) { // We need to create it
+    priv->socketTable = HashTable::create(ONE_WORD_HASH_KEYS);
   }
-  return (HashTable*)(env.groupsockPriv);
+  return priv->socketTable;
 }
 
 static Boolean unsetGroupsockBySocket(Groupsock const* groupsock) {
@@ -477,8 +476,7 @@ static Boolean unsetGroupsockBySocket(Groupsock const* groupsock) {
     // Make sure "sock" is in bounds:
     if (sock < 0) break;
 
-    HashTable* sockets = getSocketTable(groupsock->env());
-    if (sockets == NULL) break;
+    HashTable*& sockets = getSocketTable(groupsock->env());
 
     Groupsock* gs = (Groupsock*)sockets->Lookup((char*)(long)sock);
     if (gs == NULL || gs != groupsock) break;
@@ -486,8 +484,8 @@ static Boolean unsetGroupsockBySocket(Groupsock const* groupsock) {
 
     if (sockets->IsEmpty()) {
       // We can also delete the table (to reclaim space):
-      delete sockets;
-      (gs->env()).groupsockPriv = NULL;
+      delete sockets; sockets = NULL;
+      reclaimGroupsockPriv(gs->env());
     }
 
     return True;
@@ -508,10 +506,8 @@ static Boolean setGroupsockBySocket(UsageEnvironment& env, int sock,
     }
 
     HashTable* sockets = getSocketTable(env);
-    if (sockets == NULL) break;
 
-    // Make sure we're not replacing an existing Groupsock
-    // That shouldn't happen
+    // Make sure we're not replacing an existing Groupsock (although that shouldn't happen)
     Boolean alreadyExists
       = (sockets->Lookup((char*)(long)sock) != 0);
     if (alreadyExists) {
@@ -536,8 +532,6 @@ static Groupsock* getGroupsockBySocket(UsageEnvironment& env, int sock) {
     if (sock < 0) break;
 
     HashTable* sockets = getSocketTable(env);
-    if (sockets == NULL) break;
-
     return (Groupsock*)sockets->Lookup((char*)(long)sock);
   } while (0);
 
