@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2015 Live Networks, Inc.  All rights reserved.
 // Transcoder for ADUized MP3 frames
 // Implementation
 
@@ -82,7 +82,7 @@ void MP3ADUTranscoder::afterGettingFrame1(unsigned numBytesRead,
   fFrameSize = TranscodeMP3ADU(fOrigADU, numBytesRead, fOutBitrate,
 			    fTo, fMaxSize, fAvailableBytesForBackpointer);
   if (fFrameSize == 0) { // internal error - bad ADU data?
-    handleClosure(this);
+    handleClosure();
     return;
   }
 
