@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2015 Live Networks, Inc.  All rights reserved.
 // A generic SIP client
 // Implementation
 
@@ -38,6 +38,18 @@ SIPClient* SIPClient
 		       verbosityLevel, applicationName);
 }
 
+void SIPClient::setUserAgentString(char const* userAgentName) {
+  if (userAgentName == NULL) return;
+
+  // Change the existing user agent header string:
+  char const* const formatStr = "User-Agent: %s\r\n";
+  unsigned const headerSize = strlen(formatStr) + strlen(userAgentName);
+  delete[] fUserAgentHeaderStr;
+  fUserAgentHeaderStr = new char[headerSize];
+  sprintf(fUserAgentHeaderStr, formatStr, userAgentName);
+  fUserAgentHeaderStrLen = strlen(fUserAgentHeaderStr);
+}
+
 SIPClient::SIPClient(UsageEnvironment& env,
 		     unsigned char desiredAudioRTPPayloadFormat,
 		     char const* mimeSubtype,
@@ -45,11 +57,13 @@ SIPClient::SIPClient(UsageEnvironment& env,
   : Medium(env),
     fT1(500000 /* 500 ms */),
     fDesiredAudioRTPPayloadFormat(desiredAudioRTPPayloadFormat),
-    fVerbosityLevel(verbosityLevel),
-    fCSeq(0), fURL(NULL), fURLSize(0),
+    fVerbosityLevel(verbosityLevel), fCSeq(0),
+    fUserAgentHeaderStr(NULL), fUserAgentHeaderStrLen(0),
+    fURL(NULL), fURLSize(0),
     fToTagStr(NULL), fToTagStrSize(0),
     fUserName(NULL), fUserNameSize(0),
-    fInviteSDPDescription(NULL), fInviteCmd(NULL), fInviteCmdSize(0){
+    fInviteSDPDescription(NULL), fInviteSDPDescriptionReturned(NULL),
+    fInviteCmd(NULL), fInviteCmdSize(0) {
   if (mimeSubtype == NULL) mimeSubtype = "";
   fMIMESubtype = strDup(mimeSubtype);
   fMIMESubtypeSize = strlen(fMIMESubtype);
@@ -90,11 +104,7 @@ SIPClient::SIPClient(UsageEnvironment& env,
     }
   }
 
-  // Set various headers to be used in each request:
-  char const* formatStr;
-  unsigned headerSize;
-
-  // Set the "User-Agent:" header:
+  // Set the "User-Agent:" header to use in each request:
   char const* const libName = "LIVE555 Streaming Media v";
   char const* const libVersionStr = LIVEMEDIA_LIBRARY_VERSION_STRING;
   char const* libPrefix; char const* libSuffix;
@@ -104,14 +114,13 @@ SIPClient::SIPClient(UsageEnvironment& env,
     libPrefix = " (";
     libSuffix = ")";
   }
-  formatStr = "User-Agent: %s%s%s%s%s\r\n";
-  headerSize
-    = strlen(formatStr) + fApplicationNameSize + strlen(libPrefix)
-    + strlen(libName) + strlen(libVersionStr) + strlen(libSuffix);
-  fUserAgentHeaderStr = new char[headerSize];
-  sprintf(fUserAgentHeaderStr, formatStr,
+  unsigned userAgentNameSize
+    = fApplicationNameSize + strlen(libPrefix) + strlen(libName) + strlen(libVersionStr) + strlen(libSuffix) + 1;
+  char* userAgentName = new char[userAgentNameSize];
+  sprintf(userAgentName, "%s%s%s%s%s",
 	  applicationName, libPrefix, libName, libVersionStr, libSuffix);
-  fUserAgentHeaderStrSize = strlen(fUserAgentHeaderStr);
+  setUserAgentString(userAgentName);
+  delete[] userAgentName;
 
   reset();
 }
@@ -243,6 +252,7 @@ char* SIPClient::invite1(Authenticator* authenticator) {
       "INVITE %s SIP/2.0\r\n"
       "From: %s <sip:%s@%s>;tag=%u\r\n"
       "Via: SIP/2.0/UDP %s:%u\r\n"
+      "Max-Forwards: 70\r\n"
       "To: %s\r\n"
       "Contact: sip:%s@%s:%u\r\n"
       "Call-ID: %u@%s\r\n"
@@ -261,7 +271,7 @@ char* SIPClient::invite1(Authenticator* authenticator) {
       + 20 + fOurAddressStrSize
       + 20
       + strlen(authenticatorStr)
-      + fUserAgentHeaderStrSize
+      + fUserAgentHeaderStrLen
       + 20
       + inviteSDPSize;
     delete[] fInviteCmd; fInviteCmd = new char[inviteCmdSize];
@@ -432,6 +442,7 @@ void SIPClient::doInviteStateTerminated(unsigned responseCode) {
   if (responseCode < 200 || responseCode > 299) {
     // We failed, so return NULL;
     delete[] fInviteSDPDescription; fInviteSDPDescription = NULL;
+    delete[] fInviteSDPDescriptionReturned; fInviteSDPDescriptionReturned = NULL;
   }
 
   // Unblock the event loop:
@@ -595,6 +606,7 @@ unsigned SIPClient::getResponseCode() {
       }
 
       bodyStart[contentLength] = '\0'; // trims any extra data
+      delete[] fInviteSDPDescriptionReturned; fInviteSDPDescriptionReturned = strDup(bodyStart);
     }
   } while (0);
 
@@ -636,6 +648,7 @@ Boolean SIPClient::sendACK() {
       "ACK %s SIP/2.0\r\n"
       "From: %s <sip:%s@%s>;tag=%u\r\n"
       "Via: SIP/2.0/UDP %s:%u\r\n"
+      "Max-Forwards: 70\r\n"
       "To: %s;tag=%s\r\n"
       "Call-ID: %u@%s\r\n"
       "CSeq: %d ACK\r\n"
@@ -677,9 +690,10 @@ Boolean SIPClient::sendBYE() {
       "BYE %s SIP/2.0\r\n"
       "From: %s <sip:%s@%s>;tag=%u\r\n"
       "Via: SIP/2.0/UDP %s:%u\r\n"
+      "Max-Forwards: 70\r\n"
       "To: %s;tag=%s\r\n"
       "Call-ID: %u@%s\r\n"
-      "CSeq: %d ACK\r\n"
+      "CSeq: %d BYE\r\n"
       "Content-Length: 0\r\n\r\n";
     unsigned cmdSize = strlen(cmdFmt)
       + fURLSize
