@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2015 Live Networks, Inc.  All rights reserved.
 // An class that can be used to create (possibly multiple) 'replicas' of an incoming stream.
 // Implementation.
 
@@ -38,6 +38,7 @@ private:
 private:
   StreamReplicator& fOurReplicator;
   int fFrameIndex; // 0 or 1, depending upon which frame we're currently requesting; could also be -1 if we've stopped playing
+  Boolean fDeliveryInProgress;
 
   // Replicas that are currently awaiting data are kept in a (singly-linked) list:
   StreamReplica* fNext;
@@ -68,7 +69,7 @@ FramedSource* StreamReplicator::createStreamReplica() {
 
 void StreamReplicator::getNextFrame(StreamReplica* replica) {
   if (fInputSourceHasClosed) { // handle closure instead
-    FramedSource::handleClosure(replica);
+    replica->handleClosure();
     return;
   }
 
@@ -107,6 +108,9 @@ void StreamReplicator::deactivateStreamReplica(StreamReplica* replicaBeingDeacti
   if (fNumReplicas == 0) fprintf(stderr, "StreamReplicator::deactivateStreamReplica() Internal Error!\n"); // should not happen
   --fNumActiveReplicas;
 
+  if (replicaBeingDeactivated->fDeliveryInProgress) --fNumDeliveriesMadeSoFar;
+    // hack in case we're called while in the middle of a frame delivery to us
+
   // Check whether the replica being deactivated is the 'master' replica, or is enqueued awaiting a frame:
   if (replicaBeingDeactivated == fMasterReplica) {
     // We need to replace the 'master replica', if we can:
@@ -142,22 +146,40 @@ void StreamReplicator::deactivateStreamReplica(StreamReplica* replicaBeingDeacti
       }
     }
   } else {
-    // The replica that's being removed was not our 'master replica', but make sure it's not on any of our queues:
-    for (StreamReplica*& r1 = fReplicasAwaitingCurrentFrame; r1 != NULL; r1 = r1->fNext) {
-      if (r1 == replicaBeingDeactivated) {
-	r1 = replicaBeingDeactivated->fNext;
+    // The replica that's being removed was not our 'master replica', but make sure it's not on either of our queues:
+    if (fReplicasAwaitingCurrentFrame != NULL) {
+      if (replicaBeingDeactivated == fReplicasAwaitingCurrentFrame) {
+	fReplicasAwaitingCurrentFrame = replicaBeingDeactivated->fNext;
 	replicaBeingDeactivated->fNext = NULL;
-	break;
+      }
+      else {
+	for (StreamReplica* r1 = fReplicasAwaitingCurrentFrame; r1->fNext != NULL; r1 = r1->fNext) {
+	  if (r1->fNext == replicaBeingDeactivated) {
+	    r1->fNext = replicaBeingDeactivated->fNext;
+	    replicaBeingDeactivated->fNext = NULL;
+	    break;
+	  }
+	}
       }
     }
-    for (StreamReplica*& r2 = fReplicasAwaitingNextFrame; r2 != NULL; r2 = r2->fNext) {
-      if (r2 == replicaBeingDeactivated) {
-	r2 = replicaBeingDeactivated->fNext;
+    if (fReplicasAwaitingNextFrame != NULL) {
+      if (replicaBeingDeactivated == fReplicasAwaitingNextFrame) {
+	fReplicasAwaitingNextFrame = replicaBeingDeactivated->fNext;
 	replicaBeingDeactivated->fNext = NULL;
-	break;
+      }
+      else {
+	for (StreamReplica* r2 = fReplicasAwaitingNextFrame; r2->fNext != NULL; r2 = r2->fNext) {
+	  if (r2->fNext == replicaBeingDeactivated) {
+	    r2->fNext = replicaBeingDeactivated->fNext;
+	    replicaBeingDeactivated->fNext = NULL;
+	    break;
+	  }
+	}
       }
     }
   }
+
+  if (fNumActiveReplicas == 0 && fInputSource != NULL) fInputSource->stopGettingFrames(); // tell our source to stop too
 }
 
 void StreamReplicator::removeStreamReplica(StreamReplica* replicaBeingRemoved) {
@@ -167,12 +189,14 @@ void StreamReplicator::removeStreamReplica(StreamReplica* replicaBeingRemoved) {
 
   // If this was the last replica, then delete ourselves (if we were set up to do so):
   if (fNumReplicas == 0 && fDeleteWhenLastReplicaDies) {
-    delete this;
+    Medium::close(this);
     return;
   }
 
   // Now handle the replica that's being removed the same way that we would if it were merely being deactivated:
-  deactivateStreamReplica(replicaBeingRemoved);
+  if (replicaBeingRemoved->fFrameIndex != -1) { // i.e., we haven't already done this
+    deactivateStreamReplica(replicaBeingRemoved);
+  }
 }
 
 void StreamReplicator::afterGettingFrame(void* clientData, unsigned frameSize, unsigned numTruncatedBytes,
@@ -204,16 +228,16 @@ void StreamReplicator::onSourceClosure() {
   while ((replica = fReplicasAwaitingCurrentFrame) != NULL) {
     fReplicasAwaitingCurrentFrame = replica->fNext;
     replica->fNext = NULL;
-    FramedSource::handleClosure(replica);
+    replica->handleClosure();
   }
   while ((replica = fReplicasAwaitingNextFrame) != NULL) {
     fReplicasAwaitingNextFrame = replica->fNext;
     replica->fNext = NULL;
-    FramedSource::handleClosure(replica);
+    replica->handleClosure();
   }
   if ((replica = fMasterReplica) != NULL) {
     fMasterReplica = NULL;
-    FramedSource::handleClosure(replica);
+    replica->handleClosure();
   }
 }
 
@@ -226,6 +250,8 @@ void StreamReplicator::deliverReceivedFrame() {
     fReplicasAwaitingCurrentFrame = replica->fNext;
     replica->fNext = NULL;
     
+    replica->fDeliveryInProgress = True;
+
     // Assert: fMasterReplica != NULL
     if (fMasterReplica == NULL) fprintf(stderr, "StreamReplicator::deliverReceivedFrame() Internal Error 1!\n"); // shouldn't happen
     StreamReplica::copyReceivedFrame(replica, fMasterReplica);
@@ -237,6 +263,8 @@ void StreamReplicator::deliverReceivedFrame() {
 
     // Complete delivery to this replica:
     FramedSource::afterGetting(replica);
+
+    replica->fDeliveryInProgress = False;
   }
 
   if (fNumDeliveriesMadeSoFar == fNumActiveReplicas - 1 && fMasterReplica != NULL) {
@@ -273,7 +301,8 @@ void StreamReplicator::deliverReceivedFrame() {
 
 StreamReplica::StreamReplica(StreamReplicator& ourReplicator)
   : FramedSource(ourReplicator.envir()),
-    fOurReplicator(ourReplicator), fFrameIndex(-1/*we haven't started playing yet*/), fNext(NULL) {
+    fOurReplicator(ourReplicator),
+    fFrameIndex(-1/*we haven't started playing yet*/), fDeliveryInProgress(False), fNext(NULL) {
 }
 
 StreamReplica::~StreamReplica() {
@@ -285,8 +314,10 @@ void StreamReplica::doGetNextFrame() {
 }
 
 void StreamReplica::doStopGettingFrames() {
-  fFrameIndex = -1; // When we start reading again, this will tell the replicator that we were previously inactive.
-  fOurReplicator.deactivateStreamReplica(this);
+  if (fFrameIndex != -1) { // we had been activated
+    fFrameIndex = -1; // When we start reading again, this will tell the replicator that we were previously inactive.
+    fOurReplicator.deactivateStreamReplica(this);
+  }
 }
 
 void StreamReplica::copyReceivedFrame(StreamReplica* toReplica, StreamReplica* fromReplica) {
