@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2011, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2015, Live Networks, Inc.  All rights reserved
 // A program that receives and prints SDP/SAP announcements
 // (on the default SDP/SAP directory: 224.2.127.254/9875)
 
@@ -66,7 +66,7 @@ int main(int argc, char** argv) {
     }
 
     packet[packetSize] = '\0'; // just in case
-    printf((char*)(packet+8));
+    printf("%s", (char*)(packet+8));
   }
 
   return 0; // only to prevent compiler warning
